@@ -68,7 +68,7 @@ ODBC_Errors (struct jr_db_handle_s *db_handle, short last_error, char *where) {
     unsigned char  buf[250];
     unsigned char  sqlstate[15];
     unsigned char *search;
-              int  i = 0;
+    size_t  i = 0;
 
     if (where) {
         lcmaps_log(LOG_ERR, "Error detected in %s with return code: %s(%d)\n",
@@ -141,7 +141,7 @@ ODBC_Test (char *dsn, char *username, char *password)
 
 struct jr_db_handle_s *
 ODBC_Connect (char *dsn, char *username, char *password) {
-    long V_OD_erg = 0;
+    SQLRETURN V_OD_erg = 0;
     unsigned char V_OD_msg[200];
     unsigned char V_OD_stat[10];
     SQLINTEGER V_OD_err;
@@ -160,20 +160,20 @@ ODBC_Connect (char *dsn, char *username, char *password) {
 
     V_OD_erg = SQLAllocHandle(SQL_HANDLE_ENV, SQL_NULL_HANDLE, &(jr_db_handle->jobrep_henv));
     if ((V_OD_erg != SQL_SUCCESS) && (V_OD_erg != SQL_SUCCESS_WITH_INFO)) {
-            lcmaps_log(LOG_ERR, "%s: Error AllocHandle", __func__);
+            lcmaps_log(LOG_ERR, "%s: Error AllocHandle\n", __func__);
             goto failure;
     }
 
     V_OD_erg = SQLSetEnvAttr(jr_db_handle->jobrep_henv, SQL_ATTR_ODBC_VERSION, (void*)SQL_OV_ODBC3, 0);
     if ((V_OD_erg != SQL_SUCCESS) && (V_OD_erg != SQL_SUCCESS_WITH_INFO)) {
-            lcmaps_log(LOG_ERR, "%s: Error SetEnv", __func__);
+            lcmaps_log(LOG_ERR, "%s: Error SetEnv\n", __func__);
             SQLFreeHandle(SQL_HANDLE_ENV, jr_db_handle->jobrep_henv);
             goto failure;
     }
 
     V_OD_erg = SQLAllocHandle(SQL_HANDLE_DBC, jr_db_handle->jobrep_henv, &(jr_db_handle->jobrep_hdbc));
     if ((V_OD_erg != SQL_SUCCESS) && (V_OD_erg != SQL_SUCCESS_WITH_INFO)) {
-            lcmaps_log(LOG_ERR, "%s: Error AllocHDB", __func__);
+            lcmaps_log(LOG_ERR, "%s: Error AllocHDB\n", __func__);
             SQLFreeHandle(SQL_HANDLE_ENV, jr_db_handle->jobrep_henv);
             goto failure;
     }
@@ -184,7 +184,7 @@ ODBC_Connect (char *dsn, char *username, char *password) {
                           (SQLCHAR*) username, SQL_NTS,
                           (SQLCHAR*) password, SQL_NTS);
     if ((V_OD_erg != SQL_SUCCESS) && (V_OD_erg != SQL_SUCCESS_WITH_INFO)) {
-            lcmaps_log(LOG_ERR, "%s: Error SQLConnect", __func__);
+            lcmaps_log(LOG_ERR, "%s: Error SQLConnect\n", __func__);
             SQLGetDiagRec(SQL_HANDLE_DBC, jr_db_handle->jobrep_hdbc, 1,
                           V_OD_stat, &V_OD_err, V_OD_msg, sizeof(V_OD_msg), &V_OD_mlen);
             ODBC_Errors(jr_db_handle, V_OD_erg, (char *)V_OD_msg);
@@ -256,7 +256,7 @@ ODBC_Disconnect(struct jr_db_handle_s *db_handle) {
 
 int
 SQL_BeginTransaction(struct jr_db_handle_s *db_handle) {
-    int status = 0;
+    SQLRETURN status = 0;
 
     if (db_handle == NULL) {
         return -1;
@@ -357,7 +357,7 @@ SQL_BindParam(struct jr_db_handle_s *db_handle,
                     SQLUSMALLINT param_number,
                     SQLSMALLINT c_type, SQLSMALLINT sql_type,
                     SQLPOINTER data) {
-    int res;
+    SQLRETURN res;
 
     if (db_handle == NULL) {
         return -1;
@@ -393,7 +393,7 @@ Returns:
 ******************************************************************************/
 int
 SQL_Prepare (struct jr_db_handle_s *db_handle, SQLCHAR *sql) {
-    int res = 0;
+    SQLRETURN res = 0;
 
     if ((db_handle == NULL) || (sql == NULL)) {
         return -1;
@@ -436,7 +436,7 @@ Returns:
 ******************************************************************************/
 int
 SQL_Exec(struct jr_db_handle_s *db_handle) {
-    int res = 0;
+    SQLRETURN res = 0;
 
     if (db_handle == NULL) {
         return -1;
@@ -478,30 +478,29 @@ Returns:
     -1 : failure
     -2 : failure
 ******************************************************************************/
-int
+SQLRETURN
 SQL_Query(struct jr_db_handle_s *db_handle) {
-    int      res = 0;
-    int      i = 0;
-    int      colNum = 0;
-    int      rowNum = 0;
-    char     colName[255];
-    short    colType = 0;
-    short    colCType = 0;
-    UDWORD   colPrecision = 0;
-    SDWORD   colIndicator = 0;
-    short    colScale = 0;
-    short    colNullable = 0;
-    int      sts = SQL_NO_DATA_FOUND;
+    SQLRETURN res = SQL_SUCCESS;
+    SQLULEN   i = 0;
+    SQLUSMALLINT colCnt,colNum=0;
+    SQLULEN   rowCnt,rowNum=0;
+    char      colName[255];
+    short     colType = 0;
+    short     colCType = 0;
+    SQLULEN   colPrecision = 0;
+    SQLLEN    colIndicator = 0;
+    short     colScale = 0;
+    short     colNullable = 0;
 
     if (db_handle == NULL) {
-        res = -1;
-        goto cleanup;
+        res = SQL_INVALID_HANDLE;
+        goto sql_query_cleanup;
     }
 
     if (db_handle->querystate != PREPARED) {
         lcmaps_log(LOG_ERR, "%s: SQL_Query() called without SQL_Prepare()\n", __func__);
-        res = -1;
-        goto cleanup;
+        res = SQL_ERROR;
+        goto sql_query_cleanup;
     }
     /* In Query */
     db_handle->querystate = QUERY;
@@ -509,254 +508,256 @@ SQL_Query(struct jr_db_handle_s *db_handle) {
     /* SQL Execute! */
     if ((res = SQLExecute (db_handle->jobrep_hstmt)) != SQL_SUCCESS) {
         ODBC_Errors(db_handle, res, "SQLExecute");
-        res = -2;
-        goto cleanup;
+        goto sql_query_cleanup;
     }
 
     /* Got a Query, clean the ResultSet, if any is set. */
     if (db_handle->resultset != NULL) {
         lcmaps_log(LOG_ERR, "%s: The memory for the SQL resultset was not cleaned.\n", __func__);
-        res = -3;
-        goto cleanup;
+        res = SQL_ERROR;
+        goto sql_query_cleanup;
     }
 
     /* New resultset */
     db_handle->resultset = malloc(sizeof(TResultSet));
     if (db_handle->resultset == NULL) {
         lcmaps_log(LOG_ERR, "%s: Unable to allocate the resultset\n", __func__);
-        res = -3;
-        goto cleanup;
+        res = SQL_ERROR;
+        goto sql_query_cleanup;
     }
 
     /* Get column count */
     res = SQLNumResultCols(db_handle->jobrep_hstmt, &(db_handle->resultset->colCnt));
-    if (res != SQL_SUCCESS) {
+    if (res != SQL_SUCCESS || db_handle->resultset->colCnt<0) {
         ODBC_Errors(db_handle, res, "SQLNumResultCols");
-        res = -3;
-        goto cleanup;
+        goto sql_query_cleanup;
     }
 
     /* Get the row count */
     res = SQLRowCount(db_handle->jobrep_hstmt, &(db_handle->resultset->rowCnt));
-    if (res != SQL_SUCCESS) {
+    if (res != SQL_SUCCESS || db_handle->resultset->rowCnt<0) {
         ODBC_Errors(db_handle, res, "SQLRowCount");
-        res = -3;
-        goto cleanup;
+        goto sql_query_cleanup;
     }
 
-    /* Build the ResultSet Grid */
-    db_handle->resultset->columns = malloc(sizeof(TColumn) * db_handle->resultset->colCnt);
+    /* Build the ResultSet Grid *
+     * Note: colCnt is successfully retrieved, hence will be non-negative so
+     * cast explicitly to unsigned */
+    colCnt=(SQLUSMALLINT)db_handle->resultset->colCnt;
+    rowCnt=(SQLULEN)db_handle->resultset->rowCnt;
+
+    db_handle->resultset->columns = malloc(sizeof(TColumn) * colCnt);
     if (db_handle->resultset->columns == NULL) {
         lcmaps_log(LOG_ERR, "%s: Error in allocating %u bytes\n",
-                            __func__, sizeof(TColumn) * db_handle->resultset->colCnt);
-        goto done_with_filling_result_set;
+                            __func__, sizeof(TColumn) * colCnt);
+	res = SQL_ERROR;
+        goto sql_query_cleanup;
     }
-    memset(db_handle->resultset->columns, 0, sizeof(TColumn) * db_handle->resultset->colCnt);
+    memset(db_handle->resultset->columns, 0, sizeof(TColumn) * colCnt);
 
-    db_handle->resultset->data = malloc(sizeof(TRow) * db_handle->resultset->rowCnt);
+
+    db_handle->resultset->data = malloc(sizeof(TRow) * rowCnt);
     if (db_handle->resultset->data == NULL) {
         lcmaps_log(LOG_ERR, "%s: Error in allocating %u bytes\n",
-                            __func__, sizeof(TRow) * db_handle->resultset->rowCnt);
-        goto done_with_filling_result_set;
+                            __func__, sizeof(TRow) * rowCnt);
+	res = SQL_ERROR;
+        goto sql_query_cleanup;
     }
-    memset(db_handle->resultset->data, 0, sizeof(TRow) * db_handle->resultset->rowCnt);
+    memset(db_handle->resultset->data, 0, sizeof(TRow) * rowCnt);
 
-    for (i = 0; i < db_handle->resultset->rowCnt; i++) {
-        db_handle->resultset->data[i] = malloc(sizeof(TField) * db_handle->resultset->colCnt);
+    for (i = 0; i < rowCnt; i++) {
+        db_handle->resultset->data[i] = malloc(sizeof(TField) * colCnt);
         if (db_handle->resultset->data[i] == NULL) {
             lcmaps_log(LOG_ERR, "%s: Error in allocating %u bytes\n",
-                                __func__, sizeof(TField) * db_handle->resultset->colCnt);
-            goto done_with_filling_result_set;
+                                __func__, sizeof(TField) * colCnt);
+	    res = SQL_ERROR;
+            goto sql_query_cleanup;
         }
-        memset(db_handle->resultset->data[i], 0, sizeof(TField) * db_handle->resultset->colCnt);
+        memset(db_handle->resultset->data[i], 0, sizeof(TField) * colCnt);
     }
 
     /* Let's get us a dataset */
-    rowNum = 0;
-loop_for_a_new_row:
-    sts = SQLFetch (db_handle->jobrep_hstmt);
-    if (sts == SQL_NO_DATA_FOUND) {
-        goto done_with_filling_result_set;
-    }
-    if (sts != SQL_SUCCESS) {
-        ODBC_Errors(db_handle, sts, "SQLFetch");
-        goto done_with_filling_result_set;
-    }
+    for (rowNum=0; rowNum<rowCnt; rowNum++) {
+	res = SQLFetch (db_handle->jobrep_hstmt);
+	if (res == SQL_NO_DATA_FOUND) {
+	    goto sql_query_cleanup;
+	}
+	if (res != SQL_SUCCESS) {
+	    ODBC_Errors(db_handle, res, "SQLFetch");
+	    goto sql_query_cleanup;
+	}
 
-    /* Read a row, column by column */
-    for (colNum = 0; colNum < db_handle->resultset->colCnt; colNum++) {
-        sts = SQLDescribeCol(db_handle->jobrep_hstmt, colNum + 1, (UCHAR *) colName, sizeof (colName),
-                            NULL, &colType, &colPrecision,
-                            &colScale, &colNullable);
-        if (sts != SQL_SUCCESS) {
-            ODBC_Errors(db_handle, sts, "SQLDescribeCol");
-            goto done_with_filling_result_set;
-        }
+	/* Read a row, column by column */
+	for (colNum = 0; colNum < colCnt; colNum++) {
+	    res = SQLDescribeCol(db_handle->jobrep_hstmt, (SQLUSMALLINT)(colNum + 1),
+				(UCHAR *) colName, sizeof (colName),
+				NULL, &colType, &colPrecision,
+				&colScale, &colNullable);
+	    if (res != SQL_SUCCESS) {
+		ODBC_Errors(db_handle, res, "SQLDescribeCol");
+		goto sql_query_cleanup;
+	    }
 
-        void *data;
-        unsigned long bufsize_bytes;
-        bufsize_bytes = 0;
-        data = NULL;
+	    void *data;
+	    SQLLEN bufsize_bytes;
+	    bufsize_bytes = 0;
+	    data = NULL;
 
-        switch(colType) {
-            case SQL_UNKNOWN_TYPE    :
-                /* Error? */
-                break;
-            case SQL_DECIMAL         :
-            case SQL_NUMERIC         :
-                colCType = SQL_C_CHAR;
-                bufsize_bytes = sizeof(char) * 2;
-                break;
-            case SQL_BIGINT          :
-            case SQL_INTEGER         :
-                colCType = SQL_C_LONG;
-                bufsize_bytes = sizeof(long);
-                break;
-            case SQL_SMALLINT        :
-                colCType = SQL_C_SHORT;
-                bufsize_bytes = sizeof(short);
-                break;
-            case SQL_FLOAT           :
-            case SQL_REAL            :
-                colCType = SQL_C_FLOAT;
-                bufsize_bytes = sizeof(float);
-                break;
-            case SQL_DOUBLE          :
-                colCType = SQL_C_DOUBLE;
-                bufsize_bytes = sizeof(double);
-                break;
+	    switch(colType) {
+		case SQL_UNKNOWN_TYPE    :
+		    /* Error? */
+		    break;
+		case SQL_DECIMAL         :
+		case SQL_NUMERIC         :
+		    colCType = SQL_C_CHAR;
+		    bufsize_bytes = sizeof(char) * 2;
+		    break;
+		case SQL_BIGINT          :
+		case SQL_INTEGER         :
+		    colCType = SQL_C_LONG;
+		    bufsize_bytes = sizeof(long);
+		    break;
+		case SQL_SMALLINT        :
+		    colCType = SQL_C_SHORT;
+		    bufsize_bytes = sizeof(short);
+		    break;
+		case SQL_FLOAT           :
+		case SQL_REAL            :
+		    colCType = SQL_C_FLOAT;
+		    bufsize_bytes = sizeof(float);
+		    break;
+		case SQL_DOUBLE          :
+		    colCType = SQL_C_DOUBLE;
+		    bufsize_bytes = sizeof(double);
+		    break;
 #if 0
-        #if (ODBCVER >= 0x0300)
-            case SQL_DATETIME        :
+	    #if (ODBCVER >= 0x0300)
+		case SQL_DATETIME        :
 
-                break;
-        #endif
+		    break;
+	    #endif
 #endif
-            case SQL_CHAR            :
-            case SQL_VARCHAR         :
-                colCType = SQL_C_CHAR;
-                bufsize_bytes = sizeof(char) * 50;
-                break;
-            default :
-                lcmaps_log(LOG_ERR, "%s: Unknown datatype of value %d\n", __func__, colType);
-                res = -3;
-                goto cleanup;
-        }
+		case SQL_CHAR            :
+		case SQL_VARCHAR         :
+		    colCType = SQL_C_CHAR;
+		    bufsize_bytes = sizeof(char) * 50;
+		    break;
+		default :
+		    lcmaps_log(LOG_ERR, "%s: Unknown datatype of value %d\n", __func__, colType);
+		    res = SQL_ERROR;
+		    goto sql_query_cleanup;
+	    }
 
 
-        /********************************
-        Start of getting data with SQLGetData() The data will be read and made
-        available in the 'data_final' field. This can be casted to what ever
-        the colCType indicates.
-        ********************************/
-        data = malloc(bufsize_bytes);
+	    /********************************
+	    Start of getting data with SQLGetData() The data will be read and made
+	    available in the 'data_final' field. This can be casted to what ever
+	    the colCType indicates.
+	    ********************************/
+	    data = malloc((size_t)bufsize_bytes);
 
-        void *data_final = NULL;
+	    void *data_final = NULL;
 
-        void *data2 = NULL;
-        void *data3 = NULL;
-        int   total_buf = 0;
-        int   offset = 0;
-        int   needs_concat = 0;
+	    void *data2 = NULL;
+	    void *data3 = NULL;
+	    size_t total_buf = 0;
+	    SQLLEN offset = 0;
+	    int   needs_concat = 0;
 
-        data_final = data;
-        do {
-            /* Fetch data for this field */
-            memset(data, 0, bufsize_bytes);
-            res = SQLGetData (db_handle->jobrep_hstmt,
-                              colNum + 1,
-                              colCType, data, bufsize_bytes,
-                              &colIndicator);
-            if (res == SQL_SUCCESS) {
-                if (needs_concat) {
-                    /* If there is more data available for a given field, continue here */
-                    total_buf += bufsize_bytes;
+	    data_final = data;
+	    do {
+		/* Fetch data for this field */
+		memset(data, 0, (size_t)bufsize_bytes);
+		res = SQLGetData (db_handle->jobrep_hstmt,
+				  (SQLUSMALLINT)(colNum + 1),
+				  colCType, data, bufsize_bytes,
+				  &colIndicator);
+		if (res == SQL_SUCCESS) {
+		    if (needs_concat) {
+			/* If there is more data available for a given field, continue here */
+			total_buf += (size_t)bufsize_bytes;
 
-                    /* Expand memory into a new buffer and copy the current buffer in it. */
-                    if (data2 == NULL) {
-                        data3 = malloc(sizeof(char) * total_buf);
-                    } else {
-                        data3 = realloc(data2, sizeof(char) * total_buf);
-                    }
-                    if (data3 == NULL) {
-                        /* Allocation error */
-                        free(data2);
-                        goto done_with_filling_result_set;
-                    }
-                    data2 = data3;
-                    memcpy((char*)data2+offset, data, bufsize_bytes);
+			/* Expand memory into a new buffer and copy the current buffer in it. */
+			if (data2 == NULL) {
+			    data3 = malloc(sizeof(char) * total_buf);
+			} else {
+			    data3 = realloc(data2, sizeof(char) * total_buf);
+			}
+			if (data3 == NULL) {
+			    /* Allocation error */
+			    free(data2);
+			    res = SQL_ERROR;
+			    goto sql_query_cleanup;
+			}
+			data2 = data3;
+			memcpy((char*)data2+offset, data, (size_t)bufsize_bytes);
 
-                    /* Move the offset with the buffer size minus 1, because the last
-                     * byte is a \0 */
-                    offset += bufsize_bytes - 1;
+			/* Move the offset with the buffer size minus 1, because the last
+			 * byte is a \0 */
+			offset += bufsize_bytes - 1;
 
-                    data_final = data2;
-                }
-                break; /* Done */
-            } else if (res != SQL_SUCCESS_WITH_INFO) {
-                ODBC_Errors(db_handle, res, "SQLGetData");
-                goto done_with_filling_result_set;
-            }
+			data_final = data2;
+		    }
+		    break; /* Done */
+		} else if (res != SQL_SUCCESS_WITH_INFO) {
+		    ODBC_Errors(db_handle, res, "SQLGetData");
+		    goto sql_query_cleanup;
+		}
 
-            /* Signal that the last read buffer needs to be concattinated as a final step */
-            needs_concat = 1;
+		/* Signal that the last read buffer needs to be concattinated as a final step */
+		needs_concat = 1;
 
-            /* If there is more data available for a given field, continue here */
-            total_buf += bufsize_bytes;
+		/* If there is more data available for a given field, continue here */
+		total_buf += (size_t)bufsize_bytes;
 
-            /* Expand memory into a new buffer and copy the current buffer in it. */
-            if (data2 == NULL) {
-                data3 = malloc(sizeof(char) * total_buf);
-            } else {
-                data3 = realloc(data2, sizeof(char) * total_buf);
-            }
-            if (data3 == NULL) {
-                /* Allocation error */
-                free(data2);
-                goto done_with_filling_result_set;
-            }
-            data2 = data3;
-            memcpy((char*)data2+offset, data, bufsize_bytes);
-            /* Move the offset with the buffer size minus 1, because the last
-             * byte is a \0 */
-            offset += bufsize_bytes - 1;
+		/* Expand memory into a new buffer and copy the current buffer in it. */
+		if (data2 == NULL) {
+		    data3 = malloc(sizeof(char) * total_buf);
+		} else {
+		    data3 = realloc(data2, sizeof(char) * total_buf);
+		}
+		if (data3 == NULL) {
+		    /* Allocation error */
+		    free(data2);
+		    res = SQL_ERROR;
+		    goto sql_query_cleanup;
+		}
+		data2 = data3;
+		memcpy((char*)data2+offset, data, (size_t)bufsize_bytes);
+		/* Move the offset with the buffer size minus 1, because the last
+		 * byte is a \0 */
+		offset += bufsize_bytes - 1;
 
-            data_final = data2;
-        } while (res == SQL_SUCCESS_WITH_INFO);
+		data_final = data2;
+	    } while (res == SQL_SUCCESS_WITH_INFO);
 
-        /* Hook up the fetched results into the TResultSet */
-        db_handle->resultset->columns[colNum].columnname     = strdup(colName);
-        db_handle->resultset->columns[colNum].type           = colType;
-        db_handle->resultset->data[rowNum][colNum].fieldname = strdup(colName);
-        db_handle->resultset->data[rowNum][colNum].type      = colCType;
-        db_handle->resultset->data[rowNum][colNum].byte_size = total_buf;
-        switch(colCType) {
-            case SQL_C_CHAR :
-                db_handle->resultset->data[rowNum][colNum].v_string = (char *)data_final;
-                break;
-            case SQL_C_LONG :
-                db_handle->resultset->data[rowNum][colNum].v_long = *((long *)data_final);
-                break;
-            case SQL_C_SHORT:
-                db_handle->resultset->data[rowNum][colNum].v_short = *((short *)data_final);
-                break;
-            case SQL_C_FLOAT:
-                db_handle->resultset->data[rowNum][colNum].v_float = *((float *)data_final);
-                break;
-            case SQL_C_DOUBLE:
-                db_handle->resultset->data[rowNum][colNum].v_double = *((double *)data_final);
-                break;
-        }
-    }
+	    /* Hook up the fetched results into the TResultSet */
+	    db_handle->resultset->columns[colNum].columnname     = strdup(colName);
+	    db_handle->resultset->columns[colNum].type           = colType;
+	    db_handle->resultset->data[rowNum][colNum].fieldname = strdup(colName);
+	    db_handle->resultset->data[rowNum][colNum].type      = colCType;
+	    db_handle->resultset->data[rowNum][colNum].byte_size = total_buf;
+	    switch(colCType) {
+		case SQL_C_CHAR :
+		    db_handle->resultset->data[rowNum][colNum].v_string = (char *)data_final;
+		    break;
+		case SQL_C_LONG :
+		    db_handle->resultset->data[rowNum][colNum].v_long = *((long *)data_final);
+		    break;
+		case SQL_C_SHORT:
+		    db_handle->resultset->data[rowNum][colNum].v_short = *((short *)data_final);
+		    break;
+		case SQL_C_FLOAT:
+		    db_handle->resultset->data[rowNum][colNum].v_float = *((float *)data_final);
+		    break;
+		case SQL_C_DOUBLE:
+		    db_handle->resultset->data[rowNum][colNum].v_double = *((double *)data_final);
+		    break;
+	    }
+	}
+    } /* row loop */
 
-    /* Next */
-    rowNum++;
-    goto loop_for_a_new_row;
-
-done_with_filling_result_set:
-    res = db_handle->resultset->rowCnt;
-
-cleanup:
+sql_query_cleanup:
     SQL_QueryClose(db_handle);
 
     return res;
@@ -794,7 +795,7 @@ Returns:
 ******************************************************************************/
 int
 SQL_QueryClose(struct jr_db_handle_s *db_handle) {
-    int sts = 0;
+    SQLRETURN sts = 0;
 
     if (db_handle == NULL) {
         return -1;
