@@ -27,9 +27,10 @@ jobrep_grid_x509IsCA(X509 *cert) {
     return 0;
 }
 
-int
+long
 jobrep_assign_userid(struct jr_db_handle_s *db_handle, STACK_OF(X509) *px509_chain, char *eec_subject_dn) {
-    int user_id = -1, cert_id = -1;
+    long user_id = -1;
+    long cert_id = -1;
     int i = 0, depth = 0;
     int found = 0;
     char *subject_DN = NULL, *issuer_DN = NULL;
@@ -214,9 +215,10 @@ fail_jobrep:
     return -1;
 }
 
-int
+long
 jobrep_get_voms_fqan_id_from_fqan(struct jr_db_handle_s *db_handle, char *voms_fqan) {
-    int voms_fqan_id = -1;
+    long voms_fqan_id = -1;
+    SQLRETURN ret;
 
     if ((db_handle == NULL) || (voms_fqan == NULL)) {
         return -1;
@@ -233,7 +235,8 @@ jobrep_get_voms_fqan_id_from_fqan(struct jr_db_handle_s *db_handle, char *voms_f
         goto fail_jobrep;
     }
     /* The VOMS FQAN must exist in the database! */
-    if (SQL_Query(db_handle) < 0) {
+    ret=SQL_Query(db_handle);
+    if (!SQL_SUCCEEDED(ret)) {
         lcmaps_log(LOG_ERR, "%s: Failed to execute the query to fetch the voms_fqan_id from the voms_fqans.\n", __func__);
         goto fail_jobrep;
     }
@@ -263,11 +266,14 @@ fail_jobrep:
     return voms_fqan_id;
 }
 
+/* Next function is not used, commenting out */
+#if 0
 int
 jobrep_get_unix_uid_id_from_uid(struct jr_db_handle_s *db_handle, uid_t uid, char *uid_name) {
-    int unix_uid_id = -1;
+    long unix_uid_id = -1;
+    SQLRETURN ret;
 
-    if ((db_handle == NULL) || (uid < 0)) {
+    if (db_handle == NULL) {
         return -1;
     }
 
@@ -292,7 +298,8 @@ jobrep_get_unix_uid_id_from_uid(struct jr_db_handle_s *db_handle, uid_t uid, cha
         }
     }
     /* The (Unix) UID must exist in the database! */
-    if (SQL_Query(db_handle) < 0) {
+    ret=SQL_Query(db_handle);
+    if (!SQL_SUCCEEDED(ret)) {
         lcmaps_log(LOG_ERR, "%s: Failed to execute the query to fetch the unix_uid_id from the unix_uids.\n", __func__);
         goto fail_jobrep;
     }
@@ -321,12 +328,14 @@ fail_jobrep:
     SQL_QueryCloseAndClean(db_handle);
     return unix_uid_id;
 }
+#endif
 
-int
+long
 jobrep_get_unix_gid_id_from_gid(struct jr_db_handle_s *db_handle, gid_t gid, char *gid_name) {
-    int unix_gid_id = -1;
+    long unix_gid_id = -1;
+    SQLRETURN ret;
 
-    if ((db_handle == NULL) || (gid < 0)) {
+    if (db_handle == NULL) {
         return -1;
     }
 
@@ -351,7 +360,8 @@ jobrep_get_unix_gid_id_from_gid(struct jr_db_handle_s *db_handle, gid_t gid, cha
         }
     }
     /* The (Unix) GID must exist in the database! */
-    if (SQL_Query(db_handle) < 0) {
+    ret=SQL_Query(db_handle);
+    if (!SQL_SUCCEEDED(ret)) {
         lcmaps_log(LOG_ERR, "%s: Failed to execute the query to fetch the unix_gid_id from the unix_gids.\n", __func__);
         goto fail_jobrep;
     }
@@ -381,10 +391,10 @@ fail_jobrep:
     return unix_gid_id;
 }
 
-int
-jobrep_insert_unix_uid_voms_fqans(struct jr_db_handle_s *db_handle, int voms_fqan_id, int unix_uid_id) {
+long
+jobrep_insert_unix_uid_voms_fqans(struct jr_db_handle_s *db_handle, long voms_fqan_id, long unix_uid_id) {
     char                *reg_datetime = NULL;
-    int                  unix_uid_voms_fqan_id = -1;
+    long                 unix_uid_voms_fqan_id = -1;
 
     if ((db_handle == NULL) || (voms_fqan_id < 0) || (unix_uid_id < 0)) {
         return -1;
@@ -463,10 +473,10 @@ fail_jobrep:
     return -1;
 }
 
-int
-jobrep_insert_unix_gid_voms_fqans(struct jr_db_handle_s *db_handle, int voms_fqan_id, int unix_gid_id, int is_primary) {
+long
+jobrep_insert_unix_gid_voms_fqans(struct jr_db_handle_s *db_handle, long voms_fqan_id, long unix_gid_id, int is_primary) {
     char                *reg_datetime = NULL;
-    int                  unix_gid_voms_fqan_id = -1;
+    long                  unix_gid_voms_fqan_id = -1;
 
     if ((db_handle == NULL) || (voms_fqan_id < 0) || (unix_gid_id < 0)) {
         return -1;
@@ -553,10 +563,10 @@ fail_jobrep:
     return -1;
 }
 
-int
+long
 jobrep_create_effective_credentials_main(struct jr_db_handle_s *db_handle) {
     char *reg_datetime = NULL;
-    int   eff_cred_id = -1;
+    long eff_cred_id = -1;
 
     /*** Insert the effective_credentials ***/
     if (SQL_Prepare(db_handle,
@@ -615,14 +625,48 @@ fail_jobrep:
 }
 
 int
-jobrep_push_voms_fqans(struct jr_db_handle_s *db_handle, int eff_cred_id) {
+jobrep_push_compute_job_info(struct jr_db_handle_s *db_handle, long eff_cred_id, char *gatekeeper_jm_id) {
+    if (db_handle == NULL || eff_cred_id == -1 || gatekeeper_jm_id == NULL) {
+        return -1;
+    }
+
+    /*** Insert the effective_credentials ***/
+    if (SQL_Prepare(db_handle,
+            (unsigned char *)"insert into compute_jobs (eff_cred_id, gatekeeper_jm_id) values (?, ?)") < 0) {
+        lcmaps_log(LOG_ERR, "%s: Failed to prepare a query to insert a compute_jobs record into \"compute_jobs\"\n", __func__);
+        goto fail_jobrep;
+    }
+
+    /* Bind data to Query */
+    if (SQL_BindParam(db_handle, 1, SQL_C_LONG, SQL_INTEGER, &eff_cred_id) < 0) {
+        lcmaps_log(LOG_ERR, "%s: Unable to bind the \"eff_cred_id\" value to the query\n", __func__);
+        goto fail_jobrep;
+    }
+    if (SQL_BindParam(db_handle, 2, SQL_C_CHAR, SQL_VARCHAR, gatekeeper_jm_id) < 0) {
+        lcmaps_log(LOG_ERR, "%s: Unable to bind the \"GATEKEEPER_JM_ID\" value to the query\n", __func__);
+        goto fail_jobrep;
+    }
+
+    /* Fail on existance */
+    SQL_Exec(db_handle);
+
+    /* Clean up */
+    SQL_QueryCloseAndClean(db_handle);
+
+    return 0;
+fail_jobrep:
+    return -1;
+}
+
+int
+jobrep_push_voms_fqans(struct jr_db_handle_s *db_handle, long eff_cred_id) {
     int                  i = 0;
     lcmaps_vo_mapping_t *vo_mapping = NULL;
     int                  cnt_vo_mapping = 0;
-    int                  voms_fqan_id = -1;
-    int                  unix_gid_id = -1;
+    long                  voms_fqan_id = -1;
+    long                  unix_gid_id = -1;
     int                  is_primary = 0;
-    int                  unix_gid_voms_fqan_id = -1;
+    long                  unix_gid_voms_fqan_id = -1;
 
     if (db_handle == NULL) {
         return -1;
@@ -705,7 +749,7 @@ fail_jobrep:
 }
 
 int
-jobrep_push_effective_credential_user(struct jr_db_handle_s *db_handle, int user_id, int eff_cred_id) {
+jobrep_push_effective_credential_user(struct jr_db_handle_s *db_handle, long user_id, long eff_cred_id) {
     if ((db_handle == NULL) || (user_id < 0) || (eff_cred_id < 0)) {
         return -1;
     }
@@ -741,13 +785,13 @@ fail_jobrep:
 }
 
 int
-jobrep_push_effective_credential_unix_uid(struct jr_db_handle_s *db_handle, int unix_uid_id, int eff_cred_id) {
+jobrep_push_effective_credential_unix_uid(struct jr_db_handle_s *db_handle, long unix_uid_id, long eff_cred_id) {
     int                  i = 0;
     lcmaps_vo_mapping_t *vo_mapping = NULL;
     int                  cnt_vo_mapping = 0;
-    int                  voms_fqan_id = -1;
+    long                  voms_fqan_id = -1;
     /* int                  unix_uid_id = -1; */
-    int                  unix_uid_voms_fqan_id = -1;
+    long                  unix_uid_voms_fqan_id = -1;
 
     if ((db_handle == NULL) || (unix_uid_id < 0) || (eff_cred_id < 0)) {
         return -1;
@@ -825,7 +869,7 @@ fail_jobrep:
 }
 
 int
-jobrep_push_effective_credential_unix_gid(struct jr_db_handle_s *db_handle, int unix_gid_id, int eff_cred_id, int is_primary) {
+jobrep_push_effective_credential_unix_gid(struct jr_db_handle_s *db_handle, long unix_gid_id, long eff_cred_id, int is_primary) {
     if ((db_handle == NULL) || (unix_gid_id < 0) || (eff_cred_id < 0)) {
         return -1;
     }
@@ -865,14 +909,14 @@ fail_jobrep:
 }
 
 int
-jobrep_push_effective_credential_unix_gids(struct jr_db_handle_s *db_handle, int eff_cred_id) {
+jobrep_push_effective_credential_unix_gids(struct jr_db_handle_s *db_handle, long eff_cred_id) {
     int                    i = 0;
     gid_t                 *priGid = NULL;
     int                    cntPriGid = 0;
     gid_t                 *secGid = NULL;
     int                    cntSecGid = 0;
     struct group          *group_info = NULL;
-    int                    unix_gid_id = -1;
+    long                    unix_gid_id = -1;
 
     if ((db_handle == NULL) || (eff_cred_id < 0)) {
         return -1;
@@ -927,10 +971,10 @@ fail_jobrep:
     return -1;
 }
 
-int
+long
 jobrep_push_unix_cred(struct jr_db_handle_s *db_handle) {
     int                    i = 0;
-    int                    unix_uid_id_and_res = -1;
+    long                    unix_uid_id_and_res = -1;
     uid_t                 *uid = NULL;
     int                    cntUid = 0;
     gid_t                 *priGid = NULL;
@@ -1192,7 +1236,8 @@ jobrep_get_serialnumber_as_string(X509 *cert) {
     ASN1_INTEGER   *cert_Serial = NULL;
     unsigned char  *serialNumberDER = NULL, *temp = NULL, *serialStr = NULL;
     char           *subject_DN = NULL;
-    int             serialLen, len, j;
+    size_t          len;
+    int             j,serialLen;
 
     if (cert == NULL)
         return NULL;
@@ -1221,7 +1266,8 @@ jobrep_get_serialnumber_as_string(X509 *cert) {
         return NULL;
     }
 
-    temp = serialNumberDER = malloc(serialLen);
+    /* Note: serialLen is int and >0 */
+    temp = serialNumberDER = malloc((size_t)serialLen);
     if (serialNumberDER == NULL) {
         lcmaps_log(LOG_DEBUG, "%s: Could not allocate %d bytes\n", serialLen);
         return NULL;
@@ -1230,7 +1276,7 @@ jobrep_get_serialnumber_as_string(X509 *cert) {
     serialLen = i2c_ASN1_INTEGER(cert_Serial, &temp);
 
     /* Allocate as a Hex decimal + null-terminator */
-    len = serialLen * 2 + 1;
+    len = (size_t)serialLen * 2 + 1;
     serialStr = malloc(len);
     if (serialStr == NULL) {
         lcmaps_log(LOG_DEBUG, "%s: Could not allocate %d bytes\n", len);
