@@ -303,10 +303,10 @@ static int plugin_run_or_verify(
     struct jr_db_handle_s *db_handle = NULL;
     void                  *lcmaps_dummy_arg_val = NULL;
     STACK_OF(X509)        *px509_chain = NULL;
-    char                  *dn = NULL;
-    int                    user_id = -1;
-    int                    unix_uid_id = -1;
-    int                    eff_cred_id = -1;
+    char                  *dn = NULL, *gatekeeper_jm_id;
+    long                    user_id = -1;
+    long                    unix_uid_id = -1;
+    long                    eff_cred_id = -1;
 
     /* The beginning */
     if (lcmaps_mode != PLUGIN_RUN)  {
@@ -322,7 +322,7 @@ static int plugin_run_or_verify(
                                 logstr, jr_db_dsn, jr_db_user, jr_db_pass ? "a password" : "no password");
             goto fail_jobrep;
         } else {
-            lcmaps_log(LOG_INFO, "%s: Succeeded to connected to DSN \"%s\" with user \"%s\" and %s\n",
+            lcmaps_log(LOG_INFO, "%s: Succeeded to connect to DSN \"%s\" with user \"%s\" and %s\n",
                                  logstr, jr_db_dsn, jr_db_user, jr_db_pass ? "a password" : "no password");
         }
 
@@ -365,7 +365,8 @@ static int plugin_run_or_verify(
     }
 
     /* Get the DN */
-    if ((dn = *(char **)lcmaps_getArgValue("user_dn", "char *", argc, argv))) {
+    lcmaps_dummy_arg_val = lcmaps_getArgValue("user_dn", "char *", argc, argv);
+    if (lcmaps_dummy_arg_val && (dn = *(char **)lcmaps_dummy_arg_val)) {
         lcmaps_log_debug(5,"%s: found dn: %s\n", logstr, dn);
     } else {
         lcmaps_log_debug(1,"%s: could not get value of dn !\n", logstr);
@@ -392,6 +393,18 @@ static int plugin_run_or_verify(
         goto fail_jobrep;
     }
 
+    /* The GATEKEEPER_JM_ID is set in the file:
+     * gt5.2.2-all-source-installer/source-trees/gatekeeper/source/globus_gatekeeper.c:1473:
+     */
+    gatekeeper_jm_id = getenv("GATEKEEPER_JM_ID");
+    if (gatekeeper_jm_id) {
+        lcmaps_log(LOG_DEBUG, "%s: The $GATEKEEPER_JM_ID is: %s\n", logstr, gatekeeper_jm_id);
+        if (jobrep_push_compute_job_info(db_handle, eff_cred_id, gatekeeper_jm_id) < 0) {
+            lcmaps_log(LOG_ERR, "%s: Failed to insert compute job information.\n", logstr);
+            goto fail_jobrep;
+        }
+    }
+
     /* Push the VOMS FQANs from LCMAPS into the DB and hook them up to the Effective Credential ID */
     if (jobrep_push_voms_fqans(db_handle, eff_cred_id) < 0) {
         lcmaps_log(LOG_ERR, "%s: Failed to insert VOMS FQANs\n", logstr);
@@ -426,6 +439,9 @@ static int plugin_run_or_verify(
 
     /* Disconnect from the database */
     ODBC_Disconnect(db_handle);
+
+    lcmaps_log(LOG_INFO, "%s: jobrep plugin succeeded\n", logstr);
+
     return LCMAPS_MOD_SUCCESS;
 
 fail_jobrep:
